/*
   Copyright (C) 2001  Ulric Eriksson <ulric@siag.nu>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the Licence, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "sdb.h"

static void error(char *msg)
{
	perror(msg);
	exit(1);
}

static void writedata(int fd, char *b, int n)
{
	int i, j;
	for (i = 0; i < n; i += j) {
		j = write(fd, b+i, n-i);
		if (j < 0) error("Error writing data");
	}
}

static void writestring(int fd, char *b)
{
	writedata(fd, b, strlen(b));
}

static void waitforchild(int i)
{
	waitpid(-1, NULL, WNOHANG);
	signal(SIGCHLD, waitforchild);
}

static void chomp(char *p)
{
	int i;
	for (i = 0; p[i]; i++) {
		if (p[i] == '\r' || p[i] == '\n') {
			p[i] = '\0';
			return;
		}
	}
}

static int db_callback(int n, char **p, void *closure)
{
	int i;
	int fd = *((int *)closure);
	char b[10];

	if (!n) return 0;
	sprintf(b, "%d ", n);
	writestring(fd, b);		/* number of columns */
	for (i = 0; i < n; i++) {
		sprintf(b, "%d ", strlen(p[i]));
		writestring(fd, b);	/* column width */
		writestring(fd, p[i]);	/* column */
	}
	return 0;
}

static void doquery(int sock)
{
	int n;
	char u[1024];
	char q[4096];

	n = read(sock, u, sizeof u);
	if (n < 0) error("Can't read URL");
	u[n] = '\0';
	chomp(u);
	writedata(sock, " ", 1);

	n = read(sock, q, sizeof q);
	if (n < 0) error("Can't read query");
	q[n] = '\0';

	n = sdb_query(u, q, db_callback, &sock);
	writedata(sock, "0\n", 2);		/* The End */
}

int main(int argc, char *argv[])
{
	int sockfd, newsockfd, port, clilen, pid;
	struct sockaddr_in serv_addr, cli_addr;

	if (argc < 2) {
		fprintf(stderr, "Error, no port provided\n");
		exit(1);
	}
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0) error("Error opening socket");
	memset((char *) &serv_addr, 0, sizeof(serv_addr));
	port = atoi(argv[1]);
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_addr.s_addr = INADDR_ANY;
	serv_addr.sin_port = htons(port);
	if (bind(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0)
		error("Error on binding");
	listen(sockfd, 5);
	clilen = sizeof(cli_addr);
	signal(SIGCHLD, waitforchild);
	sdb_init();
	for (;;) {
		newsockfd = accept(sockfd,
				   (struct sockaddr *) &cli_addr, &clilen);
		if (newsockfd < 0) error("Error on accept");
		pid = fork();
		if (pid < 0) error("Error on fork");
		if (pid == 0) {
			close(sockfd);
			doquery(newsockfd);
			exit(0);
		} else {
			close(newsockfd);
		}
	}
	return 0;
}
