/*
A simple demo program for sdbd. Reads SQL from stdin and prints
results on stdout. Unlike sdb_client, this program does not use libsdb.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

static void error(char *msg)
{
	perror(msg);
	exit(0);
}

static void readdata(int fd, char *b, int n)
{
	int i, j;
	for (i = 0; i < n; i += j) {
		j = read(fd, b+i, n-i);
		if (j < 0) error("Error reading data");
	}
}

static void writedata(int fd, char *b, int n)
{
	int i, j;
	for (i = 0; i < n; i += j) {
		j = write(fd, b+i, n-i);
		if (j < 0) error("Error writing data");
	}
}

static void writestring(int fd, char *b)
{
	writedata(fd, b, strlen(b));
}

/* read a single char */
static int readchar(int fd)
{
	char b[1];
	readdata(fd, b, 1);
	return b[0];
}

/* read a number terminated by non-digit */
static int readno(int fd)
{
	int c, no = 0;

	for (;;) {
		c = readchar(fd);
		if (!isdigit(c)) break;
		no = 10*no + c-'0';
	}
	return no;
}

int main(int argc, char *argv[])
{
	int i, sockfd, port, ncol, colsize;
	struct sockaddr_in serv_addr;
	struct hostent *server;
	char b[4096];
	char *url;

	if (argc < 4) {
		fprintf(stderr, "usage: %s hostname port url\n", argv[0]);
		exit(0);
	}
	port = atoi(argv[2]);
	url = argv[3];
	for (;;) {
		sockfd = socket(AF_INET, SOCK_STREAM, 0);
		if (sockfd < 0) error("Error opening socket");
		server = gethostbyname(argv[1]);
		if (server == NULL) {
			fprintf(stderr, "Error, no such host\n");
			exit(0);
		}
		memset((char *) &serv_addr, 0, sizeof(serv_addr));
		serv_addr.sin_family = AF_INET;
		memcpy((char *) &serv_addr.sin_addr.s_addr,
		      (char *) server->h_addr, server->h_length);
		serv_addr.sin_port = htons(port);
		if (connect(sockfd, &serv_addr, sizeof(serv_addr)) < 0)
			error("Error connecting");
		printf("sdbd> ");
		fflush(stdout);
		if (!fgets(b, sizeof b, stdin)) break;
		writestring(sockfd, url);
		readchar(sockfd);
		writestring(sockfd, b);
		while ((ncol = readno(sockfd)) > 0) {
			for (i = 0; i < ncol; i++) {
				colsize = readno(sockfd);
				memset(b, 0, sizeof b);
				readdata(sockfd, b, colsize);
				printf("%s ", b);
			}
			printf("\n");
		}
	}
	return 0;
}
